﻿package 
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.geom.Matrix;
	import flash.events.Event;
	import flash.geom.Rectangle;
		
	
	public class Mosaic extends Sprite
	{
		var mosaicTarget:Sprite;	//被モザイク対象
		var particleSize:Number;	//モザイク粒子のサイズ

		var mosaicBitmapData:BitmapData;
		var mosaicBitmap	:Bitmap;
		var bufBitmapData	:BitmapData;
		var bufBitmap		:Bitmap;
		
		
		/**
		 * コンストラクタ
		 * @param	target	モザイクを適用するSprite
		 * @param	w		モザイクの横幅
		 * @param	h		モザイクの高さ
		 * @param	s		モザイク粒子のサイズ (省略可)
		 * @param	mask	マスクシェイプ (省略可)
		 */
		public function Mosaic(target:Sprite, w:uint, h:uint, s:Number=15, mask:Sprite=null):void
		{
			mosaicTarget = target;
			particleSize = s;
			
			//マスク設定
			if (mask) {
				this.mask = mask;			
				this.addChild(mask);
			}
			
			//ビットマップ生成
			mosaicBitmapData = new BitmapData(w, h, false);
			mosaicBitmap = new Bitmap(mosaicBitmapData);
			addChild(mosaicBitmap);
			
			bufBitmapData = new BitmapData(w/s, h/s, false);
			bufBitmap = new Bitmap(bufBitmapData);

			
			this.addEventListener(Event.ENTER_FRAME, update);
		}
		
		/**
		 * モザイククラスを解放
		 */
		public function release():void
		{
			mosaicBitmapData.dispose();
			bufBitmapData.dispose();		
		}
			
		
		/**
		 * モザイク粒子のサイズを変更
		 * @param	s	モザイク粒子のサイズ
		 */
		public function setParticleSize(s:Number):void
		{
			if (s >= 1) {
				bufBitmapData.dispose();
				bufBitmapData = new BitmapData(mosaicBitmap.width/s, mosaicBitmap.height/s, false);
				bufBitmap = new Bitmap(bufBitmapData);
				
				particleSize = s;
				
			}
		}
		
		
		/**
		 * 更新
		 * @param	e
		 */
		private function update(e:Event):void
		{
			var matrix:Matrix;
			var xs:Number = bufBitmap.width / mosaicBitmap.width;
			var ys:Number = bufBitmap.height / mosaicBitmap.height;
			
			//バッファBitmapに縮小して転送
			matrix = new Matrix(xs, 0, 0, xs,
				(mosaicTarget.x - this.x) * xs,
				(mosaicTarget.y - this.y) * ys);
			
			bufBitmapData.draw(mosaicTarget, matrix);
	
			//縮小したバッファを拡大して表示
			matrix = new Matrix(mosaicBitmap.width/bufBitmap.width, 0, 0, mosaicBitmap.height/bufBitmap.height, 0, 0);
				
			mosaicBitmapData.draw(bufBitmapData, matrix);
			
		}
		
	}
}